;;; pure-bl.el --- Library for buffer-local variables (common part, loader)

;; Copyright (C) 2000 by Project Pure.

;; Author: SHIMADA Mitsunobu <simm@irc.fan.gr.jp>
;; Keywords: PURE, buffer-local variables

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; PURE means "Primitive Universal Relay-chat Environment"
;; BL means buffer-local

;;; Code:

(defun pure-bl-make-current (alist &optional efunc)
  "Make buffer-local variables according to `alist' structured below:
'((symbol0 desc0 . value0)
  (symbol1 desc1 . value1)
  ... )"
  (let (vsym init (count 0))
    (mapcar
     '(lambda (item)
	(if (not (consp item))
	    (setq vsym nil init nil)
	  (setq vsym (car item))
	  (and (stringp vsym) (setq vsym (interm vsym)))
	  (setq init (if (consp (cdr item)) (cddr item) nil))
	;; `vsym' must be a symbol
	  (cond ((symbolp vsym)
		 (make-local-variable vsym)
		 (set vsym (eval init))
		 (setq count (1+ count)))
		((eq efunc t))
		(efunc
		 (funcall efunc vsym init))
		(t
		 (error "Cannot make a buffer-local variable: %s"
			(prin1-to-string vsym))))))
     alist)
    count))

(defun pure-bl-make-one-buffer (buf alist &optional efunc)
  "Make permanent buffer-local variables according to `alist' structured below:
'((symbol0 desc0 . value0)
  (symbol1 desc1 . value1)
  ... )"
  (save-excursion
    (set-buffer buf)
    (pure-bl-make-current alist efunc)))

(defun pure-bl-make-permanent (alist &optional efunc)
  "Make permanent buffer-local variables according to `alist' structured below:
'((symbol0 desc0 . value0)
  (symbol1 desc1 . value1)
  ... )"
  (let (vsym desc init (count 0))
    (mapcar
     '(lambda (item)
	(if (not (consp item))
	    (setq vsym nil desc nil init nil)
	  (setq vsym (car item))
	  (and (stringp vsym) (setq vsym (interm vsym)))
	  (if (consp (cdr item))
	      (setq desc (cadr item) init (cddr item))
	    (setq desc (cdr item) init nil)))
	;; `vsym' must be a symbol, `desc' must be a string or nil
	(cond ((symbolp vsym)
	       (and (stringp desc)
		    (or (boundp vsym)
			(eval (list 'defvar vsym init desc))))
	       (make-variable-buffer-local vsym)
	       (set-default vsym (eval init))
	       (setq count (1+ count)))
	      ((eq efunc t))
	      (efunc
	       (funcall efunc vsym init))
	      (t
	       (error "Cannot make a buffer-local variable: %s" (prin1-to-string vsym)))))
     alist)
    count))

;; That's all
(provide 'pure-bl)

;;; pure-bl.el ends here
