;;; pure-cs-detect.el --- Code detect function for PURE coding-system module

;; Copyright (C) 2000 by Project Pure.

;; Author: SHIMADA Mitsunobu <simm@irc.fan.gr.jp>
;; Keywords: PURE, coding-system, auto-detect

;; $Id: pure-cs-detect.el,v 1.2 2000/09/20 11:54:08 simm Exp $

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; PURE means "Primitive Universal Relay-chat Environment"
;; CS means coding-system
;; This module is derived from irchat-pj-coding-system.el

;;; Code:

(defvar pure-cs-detect-jproxy-code pure-cs-enable-mbchars
  "If non-nil, detect JProxy code.")

(defun pure-cs-detect-default-string (string)
  "Detect coding system of given `string'."
  (or (and (string-match "^[\001-\032\034-\177]*$" string)
	   'iso-8859-1)
      (and (string-match "^[\001-\177]+$" string)
	   'iso-2022-7bit-ss2)
      (and pure-cs-detect-jproxy-code
	   (string-match "\033" string) (string-match "^[\001-\177\241-\337]+$" string)
	   'iso-2022-jproxy)
      (and (string-match "\033" string) (string-match "^[\001-\177\216\217\240-\377]+$" string)
	   'ctext)))

(defun pure-cs-detect-default-region (beg end)
  "Detect coding system inner region."
  (interactive "r")
  (or (pure-cs-detect-default-string (buffer-substring beg end))))

(defun pure-cs-detect-sjis-or-default-region (beg end)
  "Detect coding system inner region.
If `flag' is non-nil, enable ISO-2022-JProxy coding system."
  (interactive "r")
  (or (pure-cs-detect-default-string (buffer-substring beg end))
      'shift_jis))

(defun pure-cs-detect-euc-jp-or-default-region (beg end)
  "Detect coding system inner region.
If `flag' is non-nil, enable ISO-2022-JProxy coding system."
  (interactive "r")
  (or (pure-cs-detect-default-string (buffer-substring beg end))
      'euc-jp))

(defun pure-cs-detect-jp-region-old (beg end)
  "Detect coding system inner region, assumes that given region is written in Japanese."
  (interactive "r")
  (or (pure-cs-detect-default-string (buffer-substring beg end))
      (catch 'detect
	(let (ch esc prev flag)
	  (save-excursion
	    (goto-char beg)
	    (while (< (point) end)
	      (setq ch (following-char))
	      (or (and (<= 128 ch) (<= ch 141)          ;; 0x80 <= ch <= 0x8d
		       (throw 'detect 'shift_jis))
		  (and (<= 144 ch) (<= ch 159)          ;; 0x90 <= ch <= 0x9f
		       (throw 'detect 'shift_jis))
		  (and (eq 'shift_jis prev)
		       (<= 161 ch) (<= ch 243)          ;; second byte of EUC kana
		       (setq prev nil flag 'euc-japan))
		  (and (eq nil prev)                    ;; first byte of EUC kana
		       (or (= 164 ch) (= 165 ch))
		       (setq prev 'euc-japan))
		  (< ch 160)                            ;; ch <= 0xa0
		  (and (eq 'euc-japan prev)
		       (throw 'detect 'euc-japan))
		  (setq prev (if prev nil 'shift_jis)
			flag (if (eq 'euc-japan flag) 'euc-japan 'shift_jis)))
	      (forward-char 1))
	    (or flag 'shift_jis))))))

(defun pure-cs-detect-jp-region (beg end)
  "Detect coding system inner region, assumes that given region is written in Japanese."
  (interactive "r")
  (or (pure-cs-detect-default-string (buffer-substring beg end))
      (let (ch (eflag nil) (sflag nil) (escore 0) (sscore 0))
	(save-excursion
	  (goto-char beg)
	  (while (< (point) end)
	    (setq ch (following-char))
	    ;; add SJIS score
	    (cond ((and sflag (<=  64 ch) (<= ch 126))
		   (setq sscore (1+ sscore) sflag nil))
		  ((and sflag (<= 128 ch) (<= ch 252))
		   (setq sscore (1+ sscore) sflag nil))
		  (sflag
		   (setq sscore (1- sscore) sflag nil))
		  ((or (= 130 ch) (= 131 ch))
		   (setq sscore (+ 2 sscore) sflag t))
		  ((and (<= 129 ch) (<= ch 159))
		   (setq sscore (1+ sscore) sflag t))
		  ((and (<= 224 ch) (<= ch 239))
		   (setq sscore (1+ sscore) sflag t))
		  (t
		   (setq sflag nil)))
	    ;; add EUC-JP score
	    (cond ((and (eq eflag '8f) (<= 161 ch) (<= ch 254))
		   (setq escore (1+ escore) eflag t))
		  ((eq eflag '8f)
		   (setq escore (1- escore) eflag nil))
		  ((and (eq eflag '8e) (<= 161 ch) (<= ch 223))
		   (setq escore (1+ escore) eflag nil))
		  ((eq eflag '8e)
		   (setq escore (1- escore) eflag nil))
		  ((and eflag (<= 161 ch) (<= ch 254))
		   (setq escore (1+ escore) eflag nil))
		  (eflag
		   (setq escore (1- escore) eflag nil))
		  ((= ch 143)
		   (setq escore (1+ escore) eflag '8f))
		  ((= ch 142)
		   (setq escore (1+ escore) eflag '8e))
		  ((= ch 255)
		   (setq escore (1- escore) eflag nil))
		  ((and (<= 127 ch) (<= ch 160))
		   (setq escore (1- escore) eflag nil))
		  ((or (= 164 ch) (= 165 ch))
		   (setq escore (+ 2 escore) eflag t))
		  ((and (<= 161 ch) (<= ch 254))
		   (setq escore (1+ escore) eflag t))
		  (t
		   (setq eflag nil)))
	    ;; next
	    (forward-char 1)))
	;;(message "SJIS: %d / EUC: %d" sscore escore)
	(if (< sscore escore) 'euc-jp 'shift_jis))))

(or (fboundp 'pure-cs-detect-region)
    (fset 'pure-cs-detect-region 'pure-cs-detect-jp-region))

;;; That's all
(provide 'pure-cs-detect)

;;; pure-cs-detect.el ends here
