;;; pure-irc-dcc-chat.el --- DCC CHAT management for PURE

;; Copyright (C) 2000 by Project Pure.

;; Author: SHIMADA Mitsunobu <simm@pure.fan.gr.jp>
;; Keywords: PURE, IRC, DCC, DCC CHAT

;; $Id: pure-irc-dcc-chat.el,v 1.1 2000/11/04 18:15:12 simm Exp $

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; PURE means "Primitive Universal Relay-chat Environment"

;;; Code:

(require 'pure-irc-dcc-common)
(require 'pure-irc-send)

(defvar pure-irc-dcc-chat-interface nil
  "Interface function for DCC CHAT.
Set function symbol, whose args are nick and message.")

;; start DCC CHAT client process
(defun pure-irc-dcc-chat-connect (dinfo &optional parent coding)
  "Start DCC CHAT connection."
  (if (not (member dinfo pure-irc-dcc-offer-list))
      (funcall pure-irc-dcc-error-function "not offered")
    (let ((host (pure-irc-dcc-ip-translate (pure-irc-dcc-info-get-host dinfo)))
	  (port (string-to-number (pure-irc-dcc-info-get-port dinfo)))
	  (nick (pure-irc-dcc-info-get-nick dinfo))
	  proc buf)
      (setq buf  (generate-new-buffer (concat " pure:DCC:CHAT:" nick))
	    proc (open-network-stream "(DCC_CHAT)" buf host port))
      (pure-irc-dcc-offer-remove dinfo)
      ;; common
      (setq pure-irc-dcc-process-list (cons proc pure-irc-dcc-process-list))
      (pure-cs-set-noconv-process proc)
      (set-process-filter   proc 'pure-irc-dcc-chat-filter)
      (set-process-sentinel proc 'pure-irc-dcc-chat-sentinel)
      (save-excursion
	(set-buffer buf)
	(pure-bl-make-current pure-irc-dcc-local-variables t)
	(setq pure-irc-dcc-status 'client
	      pure-irc-dcc-parent parent
	      pure-irc-dcc-coding coding
	      pure-irc-dcc-info   dinfo)
	(set-buffer (process-buffer parent))
	(setq pure-irc-dcc-chat-partner (cons (cons nick proc) pure-irc-dcc-chat-partner)))
      (funcall pure-irc-dcc-log-interface "DCC CHAT to %s established." nick)
      proc)))

;; set DCC CHAT server information
(defun pure-irc-dcc-chat-set-params (host port)
  "Set parameters for DCC CHAT"
  (let ((nick (pure-irc-dcc-info-get-nick)))
    (setq pure-irc-dcc-status 'wait)
    (pure-irc-dcc-info-put-host host)
    (pure-irc-dcc-info-put-port port)
    (pure-irc-dcc-info-put-size nil)
    (pure-irc-send-register pure-irc-dcc-parent pure-irc-dcc-coding
     "PRIVMSG %s :\001DCC CHAT chat %s %s\001\n" nick host port)
    (funcall pure-irc-dcc-log-interface "DCC CHAT to %s" nick)))

;; DCC CHAT message sending
(defun pure-irc-dcc-chat-send (proc msg &optional coding)
  "DCC CHAT message sending."
  (save-excursion
    (set-buffer (process-buffer proc))
    (and (pure-irc-dcc-status-is-active)
	 (pure-irc-dcc-is-chat)
	 (let ((nick (pure-irc-dcc-info-get-nick)))
	   (process-send-string proc
	    (concat (pure-cs-encode-string msg coding) "\n"))
	   nick))))

;; filter function for DCC CHAT
(defun pure-irc-dcc-chat-filter (proc msg)
  "DCC filter function for active DCC CHAT."
  (save-excursion
    (set-buffer (process-buffer proc))
    (goto-char (point-max))
    (insert msg)
    (while (and (goto-char (point-min))
		(= 0 (forward-line 1))
		(bolp))
      (save-restriction
	(narrow-to-region (point-min) (point))
	(goto-char (point-max))
	(while (or (= 10 (preceding-char)) (= 13 (preceding-char)))
	  (delete-backward-char 1))
	(pure-cs-decode-region (point-min) (point-max)
			       (pure-cs-detect-region (point-min) (point-max)))
	(funcall pure-irc-dcc-chat-interface
		 (pure-irc-dcc-info-get-nick)
		 (buffer-substring (point-min) (point-max)))
	(delete-region (point-min) (point-max))))))

;; sentinel function for DCC CHAT
(defun pure-irc-dcc-chat-sentinel (proc msg)
  "DCC CHAT sentinel function."
  (let ((stat (process-status proc))
	(buf  (process-buffer proc))
	nick)
    (save-excursion
      (set-buffer buf)
      (setq nick (pure-irc-dcc-info-get-nick)))
    (if (or (eq stat 'open) (eq stat 'run))
	(message "DCC CHAT with %s established." nick)
      (if (eq stat 'stop) nil
	(funcall pure-irc-dcc-log-interface "DCC CHAT with %s is disconnected." nick)
	(setq pure-irc-dcc-chat-partner
	      (delete (assoc nick pure-irc-dcc-chat-partner) pure-irc-dcc-chat-partner)
	      pure-irc-dcc-process-list (delete proc pure-irc-dcc-process-list))
	(set-process-buffer proc nil)
	(set-process-filter proc nil)
	(set-process-sentinel proc nil)
	(kill-buffer buf)))))

;; That's all
(provide 'pure-irc-dcc-chat)

;;; pure-irc-dcc-chat.el ends here
