;;; pure-irc-dcc-common.el --- DCC common variables and modules for PURE

;; Copyright (C) 2000 by Project Pure

;; Author: SHIMADA Mitsunobu <simm@irc.fan.gr.jp>
;; Keywords: PURE, IRC, DCC

;; $Id: pure-irc-dcc-common.el,v 1.2 2000/11/05 14:52:19 simm Exp $

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; PURE means "Primitive Universal Relay-chat Environment"

;;; Code:

(require 'pure-bl)
(require 'pure-irc-send)

(defconst pure-irc-dcc-local-variables
  '([pure-irc-dcc-status nil
     "Status of DCC process.
Server: 'set, 'wait, 'server
Client: 'connect, 'resume, 'client"]
    [pure-irc-dcc-parent nil
     "Process of IRC connection."]
    [pure-irc-dcc-coding nil
     "Coding system for IRC connection and DCC CHAT encoding."]
    [pure-irc-dcc-info nil
     "DCC information set. Consists of 6-vector:
Server: [peer-nick my-host   my-port   filename size start-time]
Client: [peer-nick peer-host peer-port filename size offer-time]
If `size' is nil, DCC CHAT request (and `filename' is \"chat\")."])
  "List of buffer-local variables.")

(defconst pure-irc-dcc-parent-local-variables
  '([pure-irc-dcc-offer-list nil
     "DCC offered list. Format of each element is same as `pure-irc-dcc-info'."]
    [pure-irc-dcc-process-list nil
     "DCC process list. Each element is process object."]
    [pure-irc-dcc-chat-partner nil
     "DCC partner association list. Each element consists '(nick . process)"])
  "List of process-local variables.")

(defvar pure-irc-dcc-log-interface nil
  "Interface function to output log message.
Set function symbol, whose arg is message.")

(defvar pure-irc-dcc-auto-get-file nil
  "*If non-nil, get a file automatically when DCC request received.")

(defvar pure-irc-dcc-use-dcc-cancel nil
  "If non-nil, send DCC CANCEL message when kill DCC offer.")

(defvar pure-irc-dcc-support-resume t
  "If non-nil, `pure-irc-dcc-program' support Send RESUME.")

;; Initialize DCC information
(defun pure-irc-dcc-initialize (logif dccif)
  "Initialize DCC information:
* Define interface functions
* Define buffer-local variables"
  (setq pure-irc-dcc-log-interface  logif
	pure-irc-dcc-chat-interface dccif)
  (pure-bl-defvar pure-irc-dcc-local-variables)
  (pure-bl-defvar pure-irc-dcc-parent-local-variables))

;; dcc-info management

(defmacro pure-irc-dcc-info-get-nick (&optional dinfo)
  (list 'aref (or dinfo 'pure-irc-dcc-info) 0))
(defmacro pure-irc-dcc-info-get-host (&optional dinfo)
  (list 'aref (or dinfo 'pure-irc-dcc-info) 1))
(defmacro pure-irc-dcc-info-get-port (&optional dinfo)
  (list 'aref (or dinfo 'pure-irc-dcc-info) 2))
(defmacro pure-irc-dcc-info-get-file (&optional dinfo)
  (list 'aref (or dinfo 'pure-irc-dcc-info) 3))
(defmacro pure-irc-dcc-info-get-size (&optional dinfo)
  (list 'aref (or dinfo 'pure-irc-dcc-info) 4))
(defmacro pure-irc-dcc-info-get-time (&optional dinfo)
  (list 'aref (or dinfo 'pure-irc-dcc-info) 5))

(defmacro pure-irc-dcc-info-put-nick (value &optional dinfo)
  (list 'aset (or dinfo 'pure-irc-dcc-info) 0 value))
(defmacro pure-irc-dcc-info-put-host (value &optional dinfo)
  (list 'aset (or dinfo 'pure-irc-dcc-info) 1 value))
(defmacro pure-irc-dcc-info-put-port (value &optional dinfo)
  (list 'aset (or dinfo 'pure-irc-dcc-info) 2 value))
(defmacro pure-irc-dcc-info-put-file (value &optional dinfo)
  (list 'aset (or dinfo 'pure-irc-dcc-info) 3 value))
(defmacro pure-irc-dcc-info-put-size (value &optional dinfo)
  (list 'aset (or dinfo 'pure-irc-dcc-info) 4 value))
(defmacro pure-irc-dcc-info-put-time (value &optional dinfo)
  (list 'aset (or dinfo 'pure-irc-dcc-info) 5 value))

(defmacro pure-irc-dcc-info-put-current-time (&optional dinfo)
  (list 'aset (or dinfo 'pure-irc-dcc-info) 5 '(current-time-string)))

(defmacro pure-irc-dcc-is-chat (&optional dinfo)
  (list 'null (list 'aref (or dinfo 'pure-irc-dcc-info) 4)))

(defmacro pure-irc-dcc-info-make (&optional nick host port file size time)
  (list 'vector nick host port file size time))

(defmacro pure-irc-dcc-status-is-active (&optional stat)
  (list 'or (list 'eq (or stat 'pure-irc-dcc-status) ''server)
	    (list 'eq (or stat 'pure-irc-dcc-status) ''client)))

;; IP address translation (DCC format -> IP address string)
(defun pure-irc-dcc-ip-translate (str)
  "Translate IP address, 32bit integer(DCC format) into \"aaa.bbb.ccc.ddd\" string."
  (if (stringp str)
      (let (delimiter num ip0 ip1 ip2 ip3)
	(if (< (length str) 9)
	    (setq num (string-to-number str)
		  ip0 (% num 256) num (/ num 256)
		  ip1 (% num 256) num (/ num 256)
		  ip2 (% num 256) ip3 (/ num 256))
	  (setq delimiter (- (length str) 8)
		num (string-to-number (substring str delimiter))
		ip0 (% num 256)
		num (+ (/ num 256) (* 390625     ; 390625 = 5 ** 8
				      (string-to-number (substring str 0 delimiter))))
		ip1 (% num 256) num (/ num 256)
		ip2 (% num 256) ip3 (/ num 256)))
	(format "%d.%d.%d.%d" ip3 ip2 ip1 ip0))))

;; SEND/CHAT offered list management
(defun pure-irc-dcc-info-complete-size (dinfo)
  "Set size to `dinfo' according to `pure-irc-dcc-offer-list'."
  (let (cinfo (dlist pure-irc-dcc-offer-list))
    (while dlist
      (setq cinfo (car dlist))
      (or (and (eq (pure-irc-dcc-info-get-nick cinfo) (pure-irc-dcc-info-get-nick dinfo))
	       (eq (pure-irc-dcc-info-get-host cinfo) (pure-irc-dcc-info-get-host dinfo))
	       (eq (pure-irc-dcc-info-get-port cinfo) (pure-irc-dcc-info-get-port dinfo))
	       (eq (pure-irc-dcc-info-get-file cinfo) (pure-irc-dcc-info-get-file dinfo))
	       (progn
		 (setq dlist nil)
		 (pure-irc-dcc-info-put-size (pure-irc-dcc-info-get-size cinfo) dinfo)
		 dinfo))
	  (setq dlist (cdr dlist))))
    dlist))

(defun pure-irc-dcc-info-complete-time (dinfo)
  "Set time to `dinfo' according to `pure-irc-dcc-offer-list'."
  (let (cinfo (dlist pure-irc-dcc-offer-list))
    (while dlist
      (setq cinfo (car dlist))
      (or (and (eq (pure-irc-dcc-info-get-nick cinfo) (pure-irc-dcc-info-get-nick dinfo))
	       (eq (pure-irc-dcc-info-get-host cinfo) (pure-irc-dcc-info-get-host dinfo))
	       (eq (pure-irc-dcc-info-get-port cinfo) (pure-irc-dcc-info-get-port dinfo))
	       (eq (pure-irc-dcc-info-get-file cinfo) (pure-irc-dcc-info-get-file dinfo))
	       (eq (pure-irc-dcc-info-get-size cinfo) (pure-irc-dcc-info-get-size dinfo))
	       (progn
		 (setq dlist nil)
		 (pure-irc-dcc-info-put-time (pure-irc-dcc-info-get-time cinfo) dinfo)
		 dinfo))
	  (setq dlist (cdr dlist))))
    dlist))

(defun pure-irc-dcc-offer-add (dinfo)
  "Add DCC offered list, called by handler.
If file is non-nil, DCC file offered.
If file is nil (and size is also nil), DCC chat offered."
  (or (member dinfo pure-irc-dcc-offer-list)
      (setq pure-irc-dcc-offer-list (cons dinfo pure-irc-dcc-offer-list))))

(defun pure-irc-dcc-offer-remove (dinfo)
  "Remove DCC offered list, called by handler."
  (and (member dinfo pure-irc-dcc-offer-list)
       (setq pure-irc-dcc-offer-list (delete dinfo pure-irc-dcc-offer-list))))

(defun pure-irc-dcc-kill (dinfo &optional parent coding)
  "Remove dcc-info from offer list.
 If `pure-irc-dcc-use-dcc-cancel' is non-nil, send DCC CANCEL message."
  (or (pure-irc-dcc-info-get-time dinfo)
      (pure-irc-dcc-info-complete-time dinfo))
  (if (not (member dinfo pure-irc-dcc-offer-list))
      (funcall pure-irc-dcc-error-function "not offered")
    (if pure-irc-dcc-use-dcc-cancel
	(pure-irc-send-register parent coding
	 "PRIVMSG %s :\001DCC CANCEL %s %s %s\001\n"
	 (pure-irc-dcc-info-get-nick dinfo) (pure-irc-dcc-info-get-file dinfo)
	 (pure-irc-dcc-info-get-host dinfo) (pure-irc-dcc-info-get-port dinfo)))
    (pure-irc-dcc-offer-remove dinfo)))

;; kill DCC process
(defun pure-irc-dcc-process-kill-all (&optional proc quit)
  "Kill all DCC process listed in `pure-irc-dcc-process-list'"
  (let (subproc)
    (save-excursion
      (if proc (set-buffer (process-buffer proc)))
      (setq pure-irc-dcc-chat-partner nil)
      (while pure-irc-dcc-process-list
	(setq subproc                   (car pure-irc-dcc-process-list)
	      pure-irc-dcc-process-list (cdr pure-irc-dcc-process-list))
	(if quit
	    (save-excursion
	      (set-buffer (process-buffer subproc))
	      (set-process-sentinel subproc nil)))
	(delete-process subproc)))))

(defun pure-irc-dcc-process-kill-nick (nick &optional proc)
  "Kill all DCC with `nick' process listed in `pure-irc-dcc-process-list'"
  (let (delete-list dcc-nick)
    (save-excursion
      (if proc (set-buffer (process-buffer proc)))
      (setq pure-irc-dcc-chat-partner (delete nick pure-irc-dcc-chat-partner))
      (mapcar
       '(lambda (item)
	  (save-excursion
	    (set-buffer (process-buffer item))
	    (setq dcc-nick (pure-irc-dcc-info-get-nick)))
	  (if (string= nick dcc-nick)
	      (setq delete-list (cons item delete-list))))
       pure-irc-dcc-process-list)
      (while delete-list
	(delete-process (car delete-list))
	(setq pure-irc-dcc-process-list (delete (car delete-list) pure-irc-dcc-process-list)
	      delete-list (cdr delete-list))))))

;; That's all
(provide 'pure-irc-dcc-common)

;;; pure-irc-dcc-common.el ends here
