;;; pure-cs-japanese.el --- Japanese processor for PURE module -*- coding: ctext; -*-

;; Copyright (C) 2000-2001 by Project Pure.

;; Author: SHIMADA Mitsunobu <simm@pure.fan.gr.jp>
;; Keywords: PURE, coding-system, ISO-2022, JISX0201, $(BH>3Q%+%J(B

;; $Id: pure-cs-japanese.el,v 1.2 2001/06/10 14:30:26 simm Exp $

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; PURE means "Primitive Universal Relay-chat Environment"
;; CS means coding-system
;; This module is derived from pure-cs-kana.el and pure-cs-detect.el
;; (pure-cs-kana.el is derived from irchat-pj-jisx0201.el)

;;; Code:

(defvar pure-cs-japanese-verbose t
  "If non-nil, output message when non-ISO-2022-JP string was given.")

(defconst pure-cs-japanese-kana-twobytes-character-alist
  '((")I" . "$(B%,(B") ("" . "$(B%.(B") ("" . "$(B%0(B") ("" . "$(B%2(B") ("" . "$(B%4(B")-A
    (")I" . "$(B%6(B") ("" . "$(B%8(B") ("" . "$(B%:(B") ("" . "$(B%<(B") ("" . "$(B%>(B")-A
    (")I" . "$(B%@(B") ("" . "$(B%B(B") ("" . "$(B%E(B") ("" . "$(B%G(B") ("" . "$(B%I(B")-A
    (")I" . "$(B%P(B") ("" . "$(B%S(B") ("" . "$(B%V(B") ("" . "$(B%Y(B") ("" . "$(B%\(B")-A
    (")I" . "$(B%Q(B") ("" . "$(B%T(B") ("" . "$(B%W(B") ("" . "$(B%Z(B") ("" . "$(B%](B")-A
    (")I" . "$(B%t(B")))-A

(defconst pure-cs-japanese-kana-onebyte-character-alist
  '((")I" . "$(B%!(B") ("" . "$(B%#(B") ("" . "$(B%%(B") ("" . "$(B%'(B") ("" . "$(B%)(B")-A
    (")I" . "$(B%c(B") ("" . "$(B%e(B") ("" . "$(B%g(B") ("" . "$(B%C(B")-A
    (")I" . "$(B%"(B") ("" . "$(B%$(B") ("" . "$(B%&(B") ("" . "$(B%((B") ("" . "$(B%*(B")-A
    (")I" . "$(B%+(B") ("" . "$(B%-(B") ("" . "$(B%/(B") ("" . "$(B%1(B") ("" . "$(B%3(B")-A
    (")I" . "$(B%5(B") ("" . "$(B%7(B") ("" . "$(B%9(B") ("" . "$(B%;(B") ("" . "$(B%=(B")-A
    (")I" . "$(B%?(B") ("" . "$(B%A(B") ("" . "$(B%D(B") ("" . "$(B%F(B") ("" . "$(B%H(B")-A
    (")I" . "$(B%J(B") ("" . "$(B%K(B") ("" . "$(B%L(B") ("" . "$(B%M(B") ("" . "$(B%N(B")-A
    (")I" . "$(B%O(B") ("" . "$(B%R(B") ("" . "$(B%U(B") ("" . "$(B%X(B") ("" . "$(B%[(B")-A
    (")I" . "$(B%^(B") ("" . "$(B%_(B") ("" . "$(B%`(B") ("" . "$(B%a(B") ("" . "$(B%b(B")-A
    (")I" . "$(B%d(B") ("" . "$(B%f(B") ("" . "$(B%h(B")-A
    (")I" . "$(B%i(B") ("" . "$(B%j(B") ("" . "$(B%k(B") ("" . "$(B%l(B") ("" . "$(B%m(B")-A
    (")I" . "$(B%o(B") ("" . "$(B%r(B") ("" . "$(B%s(B")-A
    (")I" . "$(B!"(B") ("" . "$(B!#(B") ("" . "$(B!&(B")-A
    (")I" . "$(B!+(B") ("" . "$(B!,(B") ("" . "$(B!<(B") ("" . "$(B!V(B") ("" . "$(B!W(B")))-A

(defun pure-cs-japanese-kana-convert-region (beg end)
  "Convert JISX0201 kana code into JISX0208"
  (interactive "r")
  (let (ch)
    (save-excursion
      (goto-char beg)
      (save-restriction
	(while (re-search-forward "\\ck" end t)
	  (setq ch (char-before))
	  (cond ((and (= ch ?)I) (looking-at ""))-A
		 (delete-backward-char 1)
		 (if (looking-at ")I")-A
		     (progn
		       (delete-char 2)
		       (insert "$(B!D(B"))
		   (delete-char 1)
		   (insert "$(B!E(B")))
		((looking-at "[)I]")-A
		 (backward-char)
		 (setq ch (cdr (assoc (buffer-substring (point) (+ 2 (point)))
				      pure-cs-japanese-kana-twobytes-character-alist)))
		 (if ch
		     (progn
		       (delete-char 2)
		       (insert ch))
		   (setq ch (cdr (assoc (buffer-substring (point) (1+ (point)))
					pure-cs-japanese-kana-onebyte-character-alist)))
		   (insert (or ch " "))))
		(t
		 (setq ch (cdr (assoc (buffer-substring (1- (point)) (point))
				      pure-cs-japanese-kana-onebyte-character-alist)))
		 (delete-backward-char 1)
		 (insert (or ch " ")))))))))

;;
;; Detect Japanese SHIFT_JIS or EUC-JP
;;

(defsubst pure-cs-japanese-sjis-score (ch sflag)
  (cond
   ((and sflag (<=  64 ch) (<= ch 126))
    '(1 . nil))
   ((and sflag (<= 128 ch) (<= ch 252))
    '(1 . nil))
   (sflag
    '(-1 . nil))
   ((or (= 130 ch) (= 131 ch))
    '(2 . t))
   ((and (<= 129 ch) (<= ch 159))
    '(1 . t))
   ((and (<= 224 ch) (<= ch 239))
    '(1 . t))
   (t
    '(0 . nil))))

(defsubst pure-cs-japanese-euc-score (ch eflag)
  (cond
   ((and (eq eflag '8f) (<= 161 ch) (<= ch 254))
    '(1 . t))
   ((eq eflag '8f)
    '(-1 . nil))
   ((and (eq eflag '8e) (<= 161 ch) (<= ch 223))
    '(1 . nil))
   ((eq eflag '8e)
    '(-1 . nil))
   ((and eflag (<= 161 ch) (<= ch 254))
    '(1 . nil))
   (eflag
    '(-1 . nil))
   ((= ch 143)
    '(1 . 8f))
   ((= ch 142)
    '(1 . 8e))
   ((= ch 255)
    '(-1 . nil))
   ((and (<= 127 ch) (<= ch 160))
    '(-1 . nil))
   ((or (= 164 ch) (= 165 ch))
    '(2 . t))
   ((and (<= 161 ch) (<= ch 254))
    '(1 . t))
   (t
    '(0 . nil))))

(defun pure-cs-japanese-detect-region (beg end)
  "Detect Japanese, SHIFT_JIS or EUC-JP.
This function is one of the candidates,
 `pure-cs-detect-region' and `pure-cs-detect-string'."
  (unwind-protect
      (save-restriction
	(narrow-to-region beg end)
	(pure-cs-buffer-unibyte)
	(let (ch result (eflag nil) (sflag nil) (escore 0) (sscore 0))
	  (save-excursion
	    (goto-char beg)
	    (while (< (point) end)
	      (setq ch (following-char))
	      ;; add SJIS score
	      (setq result (pure-cs-japanese-sjis-score ch sflag)
		    sscore (+ sscore (car result))
		    sflag  (cdr result))
	      ;; add EUC-JP score
	      (setq result (pure-cs-japanese-euc-score ch eflag)
		    escore (+ escore (car result))
		    eflag  (cdr result))
	      ;; next
	      (forward-char 1)))
	  (if pure-cs-japanese-verbose
	      (message "PURE: %s code detected" (if (< sscore escore) "EUC-JP" "SHIFT JIS")))
	  (if (< sscore escore) 'euc-jp 'shift_jis)))
    (pure-cs-buffer-multibyte)))

;;; That's all
(provide 'pure-cs-japanese)

;;; pure-cs-japanese.el ends here
